'''
Author: dbliu shaxunyeman@gmail.com
Date: 2024-09-02 17:55:19
LastEditors: dbliu shaxunyeman@gmail.com
LastEditTime: 2024-09-06 17:20:16
FilePath: /magic-doc/magic_doc/libs/version.py
Description: 
'''
__version__ = "0.1.5"